
# maven構成について

- java/javac version: 1.8.0_45
- maven version: 3.1.1

```
.
├── logs
├── pom.xml
├── src
│   └── main
│       ├── java
│       │   └── com
│       │       └── kddi
│       │           └── kos
│       │               └── examples
│       │                   └── upload
│       │                       └── single
│       │                           └── UploadObjectSingleOperation.java
│       └── resources
│           ├── default.properties
│           ├── default.properties_template
│           └── logback.xml
└── tmp
```

## credentialsについて

bucketへの接続情報は`src/main/resources/default.properties_template`を参考に、`src/main/resources/default.properties`へ書き込んで下さい。

```
credentials.accesskey=アクセスキー
credentials.secretkey=シークレットキー
credentials.endpoint=エンドポイント（ex: http://61.200.164.91）
bucket.1=コピー元バケット名（必須）
bucket.2=コピー先バケット名（必須）
bucket.3=
bucket.4=
bucket.5=
```

## 実行方法

`com.kddi.kos.examples.upload.single.UploadObjectSingleOperation`の実行例

```
$ mvn compile
$ mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.upload.single.UploadObjectSingleOperation
```

各クラスのJavadocに実行例を併記しています。

## logging

- exampleはslf4jとlogbackを利用
- aws-sdk for javaはlog4jを利用
    - log4jはslf4jで依存性解決済みです

デフォルトでは下記のルールでログが生成されます。詳しくはsrc/main/resources/logback.xmlを参照して下さい。

- サンプルコードから出るメッセージ
    - log level: info
    - 対象パッケージ: `com.kddi.kos.examples`
    - 書き込み対象
        - logs/example.log
        - logs/console.log
        - コンソール
- aws-sdk-javaがリクエスト／レスポンスしているREST APIの状況メッセージ
    - log level: debug
    - 対象パッケージ: `org.apache.http.wire`
    - 書き込み対象
        - logs/aws-rest.log
- サンプル外のヘルパークラスから出るメッセージ
    - log level: debug
    - 対象パッケージ: `com.kddi.helpers`
    - 書き込み対象
        - コンソール
        - logs/console.log
- 全ログ
    - logs/error.log

### エラー発見方法

1. `logs/example.log`にサンプルコードのエラーメッセージが書き込まれているか確認する。
1. エラーメッセージが出力されている時間帯と`logs/aws-rest.log`を比較し、RESTリクエストの状況から原因を考える。

### aws-sdkのデバッグ

aws-sdkのREST内容を見るためには、 org.apache.http.wire 以下のクラスに対してログレベルをdebugに設定して下さい。下記は src/main/resources/logback.xml の一部です。

```xml
<logger
  name="org.apache.http.wire"
  level="debug">
  <appender-ref ref="AWS_SDK_REQUEST_LOG" />
</logger>
```

**注意** : PUT ObjectやGET Objectなどのファイル送信処理中にファイルの中身をログに書き出してしまいます。`tail -f logs/aws-rest.log` は応答不能になるかもしれません。

- [参考](http://docs.aws.amazon.com/ja_jp/AWSSdkDocsJava/latest//DeveloperGuide/java-dg-logging.html)

## 補足情報

### 実行時に別のcredentialsに置き換える

- 下記の様にするとcredentialsを置き換えられます。

1. `src/main/resources/second.proprties` を作成。（ファイル名は任意）
1. 下記を実施

```
$ mvn compile
$ mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.upload.single.UploadObjectSingleOperation -Dcredentials=second
```

## javadoc生成

- `./target/site/apidocs/index.html` が生成されるので参照して下さい

```
$ mvn javadoc:javadoc
```
