package com.kddi.helpers.cli;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.BucketVersioningConfiguration;
import com.amazonaws.services.s3.model.SetBucketVersioningConfigurationRequest;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.helpers.util.StopWatch;

/**
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.helpers.cli.ModifyVersioningState -Dexec.args="-b bucket -v enabled/suspended"
 * </pre>
 * 
 * @author ito
 *
 */
public class ModifyVersioningState
{
  private static final Logger log = LoggerFactory.getLogger(ModifyVersioningState.class);
  private StopWatch watch = new StopWatch();

  public static void main(String[] args)
  {
    String b;
    Boolean v = false;

    try
    {
      Options opt = new Options();
      opt.addOption("h", false, "Print help for this application");
      opt.addOption("b", true, "bucket name");
      opt.addOption("v", true, "versioning state enabled/suspended");

      DefaultParser parser = new DefaultParser();
      CommandLine cl = parser.parse(opt, args);

      if (cl.hasOption('h'))
      {
        HelpFormatter f = new HelpFormatter();
        f.printHelp("OptionsTip", opt);
        System.exit(0);
      }

      if (cl.hasOption("b"))
      {
        b = cl.getOptionValue("b");
      }
      else
      {
        b = ClientProvider.getInstance().bucketName1;
        log.debug("using default bucketName : {}", b);
      }

      if (cl.hasOption("v"))
      {
        String s = cl.getOptionValue("v");
        if (s.toUpperCase().equals("ENABLED"))
        {
          v = true;
        }
        else if (s.toUpperCase().equals("SUSPENDED"))
        {
          v = false;
        }
        else
        {
          log.warn("HELPER: v option value is invalid.");
          HelpFormatter f = new HelpFormatter();
          f.printHelp("OptionsTip", opt);
          System.exit(0);
        }
      }

      new ModifyVersioningState().exec(b, v);
    }
    catch (ParseException e)
    {
      log.error("HELPER: Error Occurs:", e);
    }
  }

  /**
   * オブジェクトとバージョンの一覧を出力する
   * 
   * @param bucketName
   * @param prefix
   */
  public void exec(String bucketName, Boolean enableVersioning)
  {
    if (bucketName == null || enableVersioning == null)
    {
      throw new IllegalArgumentException();
    }

    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    BucketVersioningConfiguration config = null; // s3Client.getBucketVersioningConfiguration(bucketName);
    // log.debug("HELPER: current versioning state : [{}]", config.getStatus());

    /*
     * if (config.isMfaDeleteEnabled()) { log.warn(
     * "MultiFactorAuthenticationDelete option enabled, so unsupported this.");
     * System.exit(0); }
     */

    if (enableVersioning)
    {
      config = new BucketVersioningConfiguration(BucketVersioningConfiguration.ENABLED);
    }
    else
    {
      config = new BucketVersioningConfiguration(BucketVersioningConfiguration.SUSPENDED);
    }
    SetBucketVersioningConfigurationRequest setBucketVersioningConfigurationRequest =
            new SetBucketVersioningConfigurationRequest(bucketName, config);

    watch.start();
    s3Client.setBucketVersioningConfiguration(setBucketVersioningConfigurationRequest);
    watch.end();

    config = s3Client.getBucketVersioningConfiguration(bucketName);
    log.debug("HELPER: migrated versioning state : [{}]", config.getStatus());
  }
}
