package com.kddi.helpers.cli;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AccessControlList;
import com.amazonaws.services.s3.model.CanonicalGrantee;
import com.amazonaws.services.s3.model.Permission;
import com.kddi.helpers.provider.ClientProvider;

/**
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.helpers.cli.SetBucketACL -Dexec.args="-b bucket -u userid -acl read,write,read_acp,write_acp"
 * </pre>
 * 
 * @author ito
 *
 */
public class SetBucketACL
{
  private static final Logger log = LoggerFactory.getLogger(SetBucketACL.class);

  public static void main(String[] args)
  {
    {
      String b = null;
      String u = null;
      String acl = null;

      try
      {
        Options opt = new Options();
        opt.addOption("h", false, "Print help for this application");
        opt.addOption("b", true, "bucket name");
        opt.addOption("u", true, "canonical user id");
        opt.addOption("acl", true, "read,write,read_acp,write_acp");

        DefaultParser parser = new DefaultParser();
        CommandLine cl = parser.parse(opt, args);

        if (cl.hasOption('h'))
        {
          HelpFormatter f = new HelpFormatter();
          f.printHelp("OptionsTip", opt);
          System.exit(0);
        }

        if (cl.hasOption("b"))
        {
          b = cl.getOptionValue("b");
        }
        else
        {
          b = ClientProvider.getInstance().bucketName1;
          log.debug("using default bucketName1 : {}", b);
        }

        if (cl.hasOption("u"))
        {
          u = cl.getOptionValue("u");
        }

        if (cl.hasOption("acl"))
        {
          acl = cl.getOptionValue("acl");
        }

        new SetBucketACL().exec(b, u, acl);
      }
      catch (ParseException e)
      {
        log.error("HELPER: Error Occurs: ", e);
      }
    }
  }

  public void exec(String _bucketName, String _userid, String _acl)
  {
    AmazonS3 client = ClientProvider.getInstance().getS3Client();
    AccessControlList acl = client.getBucketAcl(_bucketName);

    CanonicalGrantee canonical = new CanonicalGrantee(_userid);

    acl.revokeAllPermissions(canonical);

    for (String param : _acl.split(","))
    {
      if (param.toLowerCase().equals("read"))
      {
        acl.grantPermission(canonical, Permission.Read);
      }
      else if (param.toLowerCase().equals("write"))
      {
        acl.grantPermission(canonical, Permission.Write);
      }
      else if (param.toLowerCase().equals("read_acp"))
      {
        acl.grantPermission(canonical, Permission.ReadAcp);
      }
      else if (param.toLowerCase().equals("write_acp"))
      {
        acl.grantPermission(canonical, Permission.WriteAcp);
      }
    }

    client.setBucketAcl(_bucketName, acl);

    new ShowBucketACL().exec(_bucketName);
  }
}
