package com.kddi.helpers.cli;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AccessControlList;
import com.amazonaws.services.s3.model.Grant;
import com.amazonaws.services.s3.model.Owner;
import com.kddi.helpers.provider.ClientProvider;

/**
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.helpers.cli.ShowBucketACL -Dexec.args="-b bucket"
 * </pre>
 * 
 * @author ito
 *
 */
public class ShowBucketACL
{
  private static final Logger log = LoggerFactory.getLogger(ShowBucketACL.class);

  public static void main(String[] args)
  {
    {
      String b;

      try
      {
        Options opt = new Options();
        opt.addOption("h", false, "Print help for this application");
        opt.addOption("b", true, "bucket name");

        DefaultParser parser = new DefaultParser();
        CommandLine cl = parser.parse(opt, args);

        if (cl.hasOption('h'))
        {
          HelpFormatter f = new HelpFormatter();
          f.printHelp("OptionsTip", opt);
          System.exit(0);
        }

        if (cl.hasOption("b"))
        {
          b = cl.getOptionValue("b");
        }
        else
        {
          b = ClientProvider.getInstance().bucketName1;
          log.debug("using default bucketName1 : {}", b);
        }

        new ShowBucketACL().exec(b);
      }
      catch (ParseException e)
      {
        log.error("HELPER: Error Occurs: ", e);
      }
    }
  }

  public void exec(String bucketName)
  {
    AmazonS3 client = ClientProvider.getInstance().getS3Client();

    AccessControlList acl = client.getBucketAcl(bucketName);
    Owner owner = acl.getOwner();
    for (Grant g : acl.getGrantsAsList())
    {
      String id = g.getGrantee().getIdentifier();
      if (id.equals(owner.getId()))
      {
        id = owner.getDisplayName();
      }
      log.debug("<{}> [{}] has [{}]", bucketName, id, g.getPermission());
    }
  }
}
