package com.kddi.helpers.cli;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.BucketVersioningConfiguration;
import com.kddi.helpers.provider.ClientProvider;

/**
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.helpers.cli.ShowVersioningStatus -Dexec.args="-b bucket"
 * </pre>
 * 
 * @author ito
 *
 */
public class ShowVersioningStatus
{
  private static final Logger log = LoggerFactory.getLogger(ShowVersioningStatus.class);

  public static void main(String[] args)
  {
    {
      String b;

      try
      {
        Options opt = new Options();
        opt.addOption("h", false, "Print help for this application");
        opt.addOption("b", true, "bucket name");

        DefaultParser parser = new DefaultParser();
        CommandLine cl = parser.parse(opt, args);

        if (cl.hasOption('h'))
        {
          HelpFormatter f = new HelpFormatter();
          f.printHelp("OptionsTip", opt);
          System.exit(0);
        }

        if (cl.hasOption("b"))
        {
          b = cl.getOptionValue("b");
        }
        else
        {
          b = ClientProvider.getInstance().bucketName1;
          log.debug("using default bucketName : {}", b);
        }

        new ShowVersioningStatus().exec(b);
      }
      catch (ParseException e)
      {
        log.error("HELPER: Error Occurs: ", e);
      }
    }
  }

  public void exec(String bucketName)
  {
    AmazonS3 client = ClientProvider.getInstance().getS3Client();

    BucketVersioningConfiguration config = client.getBucketVersioningConfiguration(bucketName);
    String versioningStatus = config.getStatus();
    log.debug("HELPER: versioning status =[{}]", versioningStatus);
  }
}
