package com.kddi.helpers.provider;

import java.util.ResourceBundle;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.ClientConfiguration;
import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.BasicAWSCredentials;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3Client;
import com.amazonaws.services.s3.S3ClientOptions;

/**
 * SDK使用方法
 * 
 * @author ito
 *
 */
public class ClientProvider
{
  public String bucketName1;
  public String bucketName2;
  public String bucketName3;
  public String bucketName4;
  public String bucketName5;

  private AmazonS3 s3Client;
  private static ClientProvider self;
  private final String DEFAULT_PROPERTY_NAME = "default";
  private static final Logger log = LoggerFactory.getLogger(ClientProvider.class);

  public static ClientProvider getInstance()
  {
    if (self == null)
    {
      self = new ClientProvider();
    }
    return self;
  }

  private ClientProvider()
  {
    String propertyName = System.getProperty("credentials");
    if (propertyName == null)
    {
      propertyName = DEFAULT_PROPERTY_NAME;
    }

    // refs to src/main/resources/*.properties
    ResourceBundle rb = ResourceBundle.getBundle(propertyName);
    log.debug("HELPER: using resource [{}]", rb.getBaseBundleName());

    String accesskey = rb.getString("credentials.accesskey");
    String secretkey = rb.getString("credentials.secretkey");
    String endpoint = rb.getString("credentials.endpoint");
    bucketName1 = rb.getString("bucket.1");
    bucketName2 = rb.getString("bucket.2");
    bucketName3 = rb.getString("bucket.3");
    bucketName4 = rb.getString("bucket.4");
    bucketName5 = rb.getString("bucket.5");

    AWSCredentials credentials = new BasicAWSCredentials(accesskey, secretkey);
    ClientConfiguration clientConfiguration = new ClientConfiguration();
    S3ClientOptions clientOptions = new S3ClientOptions();

    if (endpoint.contains(".amazonaws.com"))
    {
      clientConfiguration.withSignerOverride("AWSS3V4SignerType");// v4
      clientOptions.setPathStyleAccess(false);// disable path style access
    }
    else
    {
      clientConfiguration.withSignerOverride("S3SignerType"); // v2
      clientOptions.setPathStyleAccess(true); // enable path style access
    }

    s3Client = new AmazonS3Client(credentials, clientConfiguration);
    s3Client.setEndpoint(endpoint);
    s3Client.setS3ClientOptions(clientOptions);
  }

  /**
   * credentials, signature type, endpointを設定したclientを返す。
   * 
   * @return AmazonS3
   */
  public AmazonS3 getS3Client()
  {
    return s3Client;
  }
}
