package com.kddi.helpers.provider;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.BucketVersioningConfiguration;
import com.amazonaws.services.s3.model.SetBucketVersioningConfigurationRequest;
import com.kddi.helpers.cli.DeleteObjects;
import com.kddi.helpers.cli.ShowObjectsVersions;
import com.kddi.helpers.creator.MockCreator;

public class UtilProvider
{
  private static final Logger log = LoggerFactory.getLogger(UtilProvider.class);
  private AmazonS3 client;

  public UtilProvider(AmazonS3 client)
  {
    log.debug("HELPER: set AmazonS3 client.");
    this.client = client;
  }

  /**
   * オブジェクトとバージョンの一覧を出力する
   * 
   * @param bucketName
   * @param prefix
   * @param isShowDeletedKey 削除マーカー付与済みkeyもリスト表示するか
   */
  public void showObjectsVersions(String bucketName, String prefix, Boolean isShowDeletedKey)
  {
    new ShowObjectsVersions().exec(bucketName, prefix, isShowDeletedKey);
  }

  /**
   * オブジェクトとバージョンの一覧を出力する
   * 
   * @param bucketName
   * @param prefix
   */
  public void showObjectsVersions(String bucketName, String prefix)
  {
    new ShowObjectsVersions().exec(bucketName, prefix, false);
  }

  /**
   * オブジェクトをバージョンを削除する
   * 
   * @param bucketName
   * @param prefix
   */
  public void deleteObjectsVersions(String bucketName, String prefix)
  {
    new DeleteObjects().exec(bucketName, prefix, DeleteObjects.OBJECT_ONLY);
  }

  /**
   * bucketのversioning statusを確認する。
   * 
   * @see MockCreator#versioning(String, Boolean)
   */
  public void versioning(String bucketName)
  {
    versioning(bucketName, null);
  }

  /**
   * bucketのversioning statusを変更確認する。
   * 
   * @param bucketName
   * @param enable Boolean true/false/null
   */
  public void versioning(String bucketName, Boolean enable)
  {
    BucketVersioningConfiguration config = null;
    if (enable != null)
    {
      config = new BucketVersioningConfiguration();
      String before_msg = "", after_msg = "";

      if (enable)
      {
        // set enable
        before_msg = "versioning enabling.. ";
        config.setStatus(BucketVersioningConfiguration.ENABLED);
        after_msg = "versioning enabled.";
      }
      else
      {
        // set suspended
        before_msg = "versioning suspending.. ";
        config.setStatus(BucketVersioningConfiguration.SUSPENDED);
        after_msg = "versioning suspended";
      }

      long startTime = System.nanoTime();
      log.debug("HELPER: {}", before_msg);
      SetBucketVersioningConfigurationRequest setBVCRequest = new SetBucketVersioningConfigurationRequest(bucketName, config);
      this.client.setBucketVersioningConfiguration(setBVCRequest);

      double elapsed_sec = (double) (System.nanoTime() - startTime) / 1000000000.0;
      log.debug("HELPER: {} elapsed : {} secs", after_msg, elapsed_sec);
    }

    // show versioning status
    config = this.client.getBucketVersioningConfiguration(bucketName);
    String status = config.getStatus();
    log.debug("HELPER: current versioning status : {}", status);
  }
}
