package com.kddi.kos.examples.delete.single;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.model.DeleteObjectRequest;
import com.kddi.helpers.creator.MockCreator;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.helpers.provider.UtilProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>オブジェクトの削除
 * <li>1 件のリクエストで 1 つのオブジェクトを削除
 * <li>オブジェクトの削除（バージョニング非対応のバケット）
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.delete.single.DeleteAnObject
 * </pre>
 * 
 * @author ito
 *
 */
public class DeleteAnObject implements IExample
{
  private static final Logger log = LoggerFactory.getLogger(DeleteAnObject.class);

  public static void main(String[] args) throws IOException
  {
    new DeleteAnObject().exec();
  }

  @Override
  public void exec()
  {
    // 1. clientの準備
    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    String bucketName = ClientProvider.getInstance().bucketName1;
    String keyName = "DeleteAnObject.txt";

    try
    {
      // サンプル外操作：1MBのファイルをtmpに作成後、指定bucketにアップロードする
      new MockCreator(s3Client).createTextFile(keyName, 1L * MockCreator.SIZE_MB).uploadTo(bucketName);
      // サンプル外操作：本サンプルで使うkeyがstorage上に存在する事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);

      // 2. ObjectStorage上にあるオブジェクトをキー名で削除する
      log.info("deleting.. Object: {}", keyName);
      s3Client.deleteObject(new DeleteObjectRequest(bucketName, keyName));
      log.info("deleted Object: {}", keyName);

      // サンプル外操作：本サンプルで使うkeyがstorage上に存在しない事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);
    }
    catch (AmazonS3Exception e)
    {
      // サービス側のリクエスト処理中エラー
      // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
      // アクセスキー間違いなど
      log.error("Error Occurs:", e);
    }
    catch (AmazonServiceException e)
    {
      // サービス側のリクエスト処理開始時エラー
      // リクエストは到達したが処理開始出来なかったケース
      log.error("Error Occurs:", e);
    }
    catch (AmazonClientException e)
    {
      // クライアント側のレスポンス処理開始エラー
      // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
      // copyPart()を実行するなど
      log.error("Error Occurs:", e);
    }
    catch (FileNotFoundException e)
    {
      log.error("Error Occurs:", e);
    }
    finally
    {
      log.info("END");
    }
  }
}
