package com.kddi.kos.examples.download;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.model.GetObjectRequest;
import com.amazonaws.services.s3.model.S3Object;
import com.kddi.helpers.creator.MockCreator;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.helpers.provider.UtilProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>オブジェクトの取得
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.download.GetObject
 * </pre>
 * 
 * @author ito
 *
 */
public class GetObject implements IExample
{
  private static final Logger log = LoggerFactory.getLogger(GetObject.class);

  public static void main(String[] args) throws IOException
  {
    new GetObject().exec();
  }

  @Override
  public void exec()
  {
    // 1. clientの準備
    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    String bucketName = ClientProvider.getInstance().bucketName1;
    String keyName = "GetObject.txt";

    try
    {
      // サンプル外操作：本サンプルで使ったkeyのみを対象にオブジェクト及びバージョン削除を試行
      new UtilProvider(s3Client).deleteObjectsVersions(bucketName, keyName);
      // サンプル外操作：1MBのファイルをtmpに作成後、指定bucketにアップロード
      new MockCreator(s3Client).createTextFile(keyName, 1L * MockCreator.SIZE_MB).uploadTo(bucketName);
      // サンプル外操作：本サンプルで使うkeyがstorage上に存在する事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);

      // 2. ObjectStorage上にあるkeyを指定して取得
      GetObjectRequest getObjectRequest = new GetObjectRequest(bucketName, keyName);
      log.info("Downloading an object: {}", getObjectRequest.getKey());
      S3Object s3object = s3Client.getObject(getObjectRequest);

      // 3. keyのファイルタイプや長さを取得
      log.info("Content-Type:  {}", s3object.getObjectMetadata().getContentType());
      log.info("Content Length: {}", s3object.getObjectMetadata().getContentLength());
      s3object.close();
    }
    catch (AmazonS3Exception e)
    {
      // サービス側のリクエスト処理中エラー
      // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
      // アクセスキー間違いなど
      log.error("Error Occurs:", e);
    }
    catch (AmazonServiceException e)
    {
      // サービス側のリクエスト処理開始時エラー
      // リクエストは到達したが処理開始出来なかったケース
      log.error("Error Occurs:", e);
    }
    catch (AmazonClientException e)
    {
      // クライアント側のレスポンス処理開始エラー
      // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
      // copyPart()を実行するなど
      log.error("Error Occurs:", e);
    }
    catch (FileNotFoundException e)
    {
      log.error("Error Occurs: {}", e);
    }
    catch (IOException e)
    {
      log.error("Error Occurs: {}", e);
    }
    finally
    {
      log.info("END");
    }
  }
}
