package com.kddi.kos.examples.list;

import java.io.IOException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.model.Bucket;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>bucketのリスト作成
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.list.ListBuckets
 * </pre>
 * 
 * @author ito
 *
 */
public class ListBuckets implements IExample
{
    private static final Logger log = LoggerFactory.getLogger(ListBuckets.class);

    public static void main(String[] args) throws IOException
    {
        new ListBuckets().exec();
    }

    @Override
    public void exec()
    {
        // 1. clientの準備
        AmazonS3 s3client = ClientProvider.getInstance().getS3Client();

        try
        {
            // 2. 契約に紐付くbucketのリストを取得
            List<Bucket> blist = s3client.listBuckets();
            for (Bucket bucket : blist)
            {
                log.info("bucket name: {}", bucket.getName());
            }
        } catch (AmazonS3Exception e)
        {
            // サービス側のリクエスト処理中エラー
            // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
            // アクセスキー間違いなど
            log.error("Error Occurs:", e);
        } catch (AmazonServiceException e)
        {
            // サービス側のリクエスト処理開始時エラー
            // リクエストは到達したが処理開始出来なかったケース
            log.error("Error Occurs:", e);
        } catch (AmazonClientException e)
        {
            // クライアント側のレスポンス処理開始エラー
            // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
            // copyPart()を実行するなど
            log.error("Error Occurs:", e);
        } finally
        {
            log.info("END");
        }
    }
}
