package com.kddi.kos.examples.list;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.model.ListObjectsRequest;
import com.amazonaws.services.s3.model.ObjectListing;
import com.amazonaws.services.s3.model.S3ObjectSummary;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>オブジェクトキーのリスト作成
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.list.ListKeys
 * </pre>
 * 
 * @author ito
 *
 */
public class ListKeys implements IExample
{
  private static final Logger log = LoggerFactory.getLogger(ListKeys.class);

  public static void main(String[] args) throws IOException
  {
    new ListKeys().exec();
  }

  @Override
  public void exec()
  {
    // 1. clientの準備
    AmazonS3 s3client = ClientProvider.getInstance().getS3Client();
    String bucketName = ClientProvider.getInstance().bucketName1;

    try
    {
      // 2. ObjectStorage上にあるオブジェクトのリストを取得表示
      log.info("Listing objects");

      ListObjectsRequest listObjectsRequest = new ListObjectsRequest().withBucketName(bucketName);
      ObjectListing objectListing;
      do
      {
        // 2-1.最初の1000件を取得
        objectListing = s3client.listObjects(listObjectsRequest);
        for (S3ObjectSummary objectSummary : objectListing.getObjectSummaries())
        {
          log.info("{} {}byte", objectSummary.getKey(), objectSummary.getSize());
        }
        listObjectsRequest.setMarker(objectListing.getNextMarker());
      }
      while (objectListing.isTruncated()); // 2-2. もし1001件が存在するなら再実行
    }
    catch (AmazonS3Exception e)
    {
      // サービス側のリクエスト処理中エラー
      // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
      // アクセスキー間違いなど
      log.error("Error Occurs:", e);
    }
    catch (AmazonServiceException e)
    {
      // サービス側のリクエスト処理開始時エラー
      // リクエストは到達したが処理開始出来なかったケース
      log.error("Error Occurs:", e);
    }
    catch (AmazonClientException e)
    {
      // クライアント側のレスポンス処理開始エラー
      // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
      // copyPart()を実行するなど
      log.error("Error Occurs:", e);
    }
    finally
    {
      log.info("END");
    }
  }
}
