package com.kddi.kos.examples.presignedUrl;

import java.io.FileNotFoundException;
import java.net.URL;
import java.util.Date;

import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.HttpMethod;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.kddi.helpers.creator.MockCreator;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.helpers.provider.UtilProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>署名付きURL
 * <li>GET Object
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.presignedUrl.GetObject
 * </pre>
 * 
 * @author ito
 *
 */
public class GetObject implements IExample
{
  private static final Logger log = LoggerFactory.getLogger(GetObject.class);

  @Override
  public void exec()
  {
    try
    {
      // 1. clientの準備
      AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
      String bucketName = ClientProvider.getInstance().bucketName1;
      String keyName = "presignedUrl.txt";

      // サンプル外操作：本サンプルの実行前状態を確認
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);
      // サンプル外操作：本サンプルで使ったkeyのみを対象にオブジェクト及びバージョン削除を試行
      new UtilProvider(s3Client).deleteObjectsVersions(bucketName, keyName);
      // サンプル外操作：1MBのファイルをtmpに作成
      new MockCreator(s3Client).createTextFile(keyName, 1L * MockCreator.SIZE_MB).uploadTo(bucketName);
      // サンプル外操作：本サンプルで使うkeyがstorage上に存在しない事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);

      // 2. 10日後に失効する認証済みGET URLを取得
      DateTime today = new DateTime();
      Date after10day = today.plusDays(10).toDate();
      URL url = s3Client.generatePresignedUrl(bucketName, keyName, after10day, HttpMethod.GET);

      log.info("REST: GET /{}/{}", bucketName, keyName);
      log.info("GET PresignedURL: {}", url.toString());
    }
    catch (AmazonS3Exception e)
    {
      // サービス側のリクエスト処理中エラー
      // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
      // アクセスキー間違いなど
      log.error("Error Occurs:", e);
    }
    catch (AmazonServiceException e)
    {
      // サービス側のリクエスト処理開始時エラー
      // リクエストは到達したが処理開始出来なかったケース
      log.error("Error Occurs:", e);
    }
    catch (AmazonClientException e)
    {
      // クライアント側のレスポンス処理開始エラー
      // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
      // copyPart()を実行するなど
      log.error("Error Occurs:", e);
    }
    catch (FileNotFoundException e)
    {
      log.error("Error Occurs:", e);
    }
    finally
    {
      log.info("END");
    }
  }

  public static void main(String[] args)
  {
    new GetObject().exec();
  }

}
